<?php
/**
 * "Платёжная система ВТБ" 
 * Контроллер отвечает за проверку статуса оплаты и редирект на страницу соответствующую статусу
 *
 * @author    VTB <acquiring_support@vtb.ru>
 * @copyright Since 2023 VTB
 */

/**
 * This Controller receive customer after approval on bank payment page
 */
class vtbpayReturnModuleFrontController extends ModuleFrontController
{
    /**
     * Обработчик POST-запроса.
     *
     * Этот метод обрабатывает POST-запрос после возврата клиента с банковской страницы оплаты.
     * Здесь происходит валидация данных, изменение статуса заказа и редирект на соответствующую страницу.
     *
     * @throws Exception Исключение выбрасывается в случае ошибки.
     * @return void
     */
    public function postProcess()
    {
        $this->module->setVtbpayLogger();
        $orderId = $_GET['orderId'] ?? '';

        try {
            $this->validate($orderId);
            
            $paymentStatus = $this->getPaymentStatus($orderId);
            
            $this->changePaymentStatus($orderId, $paymentStatus);
            
            Tools::redirect($this->getRedirectPage($orderId, $paymentStatus));
            
        } catch (\Exception | VtbPayException $e) {
            // Обрабатываем исключение и логируем ошибку
            $this->handleErrorAndRedirect($e, $orderId);
        }
    }
    
    
    /**
     * Проверка и валидация данных.
     *
     * @param int $orderId Идентификатор заказа.
     * @throws Exception Исключение выбрасывается в случае ошибки.
     * @return void
     */
    private function validate($orderId)
    {
        if (empty($orderId)) {
            throw new \Exception($this->module->l('Идентификатор заказа пуст.'));
        }
      
        if (!$this->isPaymentOptionAvailable()) {
            throw new \Exception($this->module->l('Способ оплаты не доступен.'));
        }
        
        if (!Validate::isLoadedObject($this->context->customer)) {
            throw new \Exception($this->module->l('Клиент не найден.'));
        }
        
        $order = new Order($orderId);
        if (!Validate::isLoadedObject($order)) {
            throw new \Exception($this->module->l('Заказ не найден.'));
        }
    }
    
    
    /**
     * Изменение статуса платежа в заказе.
     *
     * @param int $orderId Идентификатор заказа.
     * @param string $paymentStatus Статус заказа.
     * @return bool Возвращает `true` в случае успеха, иначе `false`.
     */
    private function changePaymentStatus($orderId, $paymentStatus)
    {
        $newOrderHistory = new OrderHistory();
        $newOrderHistory->id_order = (int)$orderId;
        $newOrderHistory->id_employee = 0; // ID сотрудника (0 для автоматической записи)

        if ($paymentStatus === 'PAID') {
            $newOrderHistory->changeIdOrderState((int) Configuration::get('PS_OS_PAYMENT'), $orderId);
        } 
        else {
            $newOrderHistory->changeIdOrderState((int) Configuration::get('PS_OS_ERROR'), $orderId);
        }
        
        if (!$newOrderHistory->addWithemail(true)) {
            throw new \Exception($this->module->l('Статус платежа не изменился.'));
        }
    }
    
    
    /**
     * Получение URL для редиректа.
     *
     * @param int $orderId Идентификатор заказа.
     * @param string $paymentStatus Статус платежа.
     * @return string URL для редиректа.
     */
    private function getRedirectPage($orderId, $paymentStatus)
    {
        $redirectPage = $this->context->link->getPageLink(
            'order-detail', 
            true, 
            (int) $this->context->language->id, [
                'id_cart' => (int) $this->context->cart->id,
                'id_module' => (int) $this->module->id,
                'id_order' => (int) $orderId,
                'key' => $this->context->customer->secure_key,
            ]
        );
        
        $configure = $this->module->configure;
        
        if (
            $paymentStatus === 'PAID' && 
            isset($configure['VTBPAY_SUCCESSFUL_PAYMENT_PAGE']) &&
            !empty($configure['VTBPAY_SUCCESSFUL_PAYMENT_PAGE'])
        ) {
            $redirectPage = $configure['VTBPAY_SUCCESSFUL_PAYMENT_PAGE'];
        } 
        elseif (
            $paymentStatus !== 'PAID' && 
            isset($configure['VTBPAY_FAILED_PAYMENT_PAGE']) &&
            !empty($configure['VTBPAY_FAILED_PAYMENT_PAGE'])
        ) {
            $redirectPage = $configure['VTBPAY_FAILED_PAYMENT_PAGE'];
        }
        
        return $redirectPage;
    }
    
    
    /**
     * Получение статуса платежа из внешнего источника.
     *
     * @param int $orderId Идентификатор заказа.
     * @return string Возвращает статус платежа.
     */
    private function getPaymentStatus($orderId): string
    {
        // Logging input
        $this->module->logger->debug(__FUNCTION__ . ' > getOrderInfo - INPUT: ', [
            'arguments' => [
                'order_id' => $orderId
            ]
        ]);
        
        $response = $this->module->getVtbApi()->getOrderInfo($orderId);
        
        // Logging output
        $this->module->logger->debug(__FUNCTION__ . ' > getOrderInfo - OUTPUT: ', [
            'response' => $response
        ]);

        return $response['object']['status']['value'] ?? '';
    }
    

    /**
     * Проверка доступности опции оплаты.
     *
     * @return bool Возвращает `true`, если опция оплаты доступна, иначе `false`.
     */
    private function isPaymentOptionAvailable()
    {
        if (!empty($modules = Module::getPaymentModules())) {
            foreach ($modules as $module) {
                if (isset($module['name']) && $this->module->name === $module['name']) {
                    return true;
                }
            }
        }

        return false;
    }
    
    
    /**
     * Обработка ошибки и перенаправление.
     *
     * @param Exception|VtbPayException $error_obj Объект ошибки.
     * @param string $orderId Идентификатор заказа.
     * @return void
     */
    private function handleErrorAndRedirect($error_obj, $orderId)
    {
        // Handle exception and log error
        $context = [];
        if (method_exists($error_obj, 'getContext')) $context = $error_obj->getContext();
        
        $this->module->logger->error(sprintf(
            __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;', 
            $error_obj->getMessage(), 
            $orderId ?: ''
        ), $context);
        
        $this->errors[] = $error_obj->getMessage();
        $this->redirectWithNotifications($this->context->link->getPageLink(
            'cart',
            true,
            (int) $this->context->language->id, [
                'action' => 'show',
            ]
        ));
    }
}
