<?php
/**
 * Модуль для интеграции с платежной системой ВТБ.
 *
 * @author    VTB <acquiring_support@vtb.ru>
 * @copyright Since 2023 VTB
 */
 
if (!defined('_PS_VERSION_')) exit;
 
require_once 'vendor/autoload.php';

use PrestaShop\PrestaShop\Core\Payment\PaymentOption,
    \Vtbpay\Classes\Api\VtbApi,
    \Vtbpay\Classes\Common\VtbPayLogger,
    \Vtbpay\Classes\Exception\VtbPayException;
    

class VtbPay extends PaymentModule
{
    /**
     * @var array Массив с конфигурацией модуля.
     */
    public $configure;
    
    /**
     * @var VtbApi|null Объект для работы с API ВТБ.
     */
    private $vtb_api_obj = null;
    
    /**
     * @var VtbPayLogger|null Объект для логирования.
     */
    public $logger;


    /**
     * В конструкторе инициализируем настройки модуля.
     */
    public function __construct() 
    {
        $this->name = 'vtbpay';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.2';
        $this->author = 'VTB';
        $this->author_uri = 'https://acquiring.vtb.ru';
        $this->logo = __DIR__ . '/videws/img/logotype.png';
        $this->need_instance = 1;

        $this->controllers = [
            'payment',
            'return',
        ];

        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Платежная система ВТБ');
        $this->description = $this->l('Расплачивайтесь любой картой банка без комиссии.');

        $this->confirmUninstall = $this->l('Вы уверены, что хотите удалить этот модуль?');

        $this->ps_versions_compliancy = [
            'min' => '1.7',
            'max' => _PS_VERSION_
        ];

        $this->configure = $this->getConfigFormValues();
    }
    
    
    /**
     * Получает объект для работы с API ВТБ.
     *
     * @return VtbApi Объект для работы с API ВТБ.
     */
    public function getVtbApi(): VtbApi
    {
        if (empty($this->vtb_api_obj)) {
            $config = $this->configure;
            $this->vtb_api_obj = new VtbApi(
                $config['VTBPAY_CLIENT_ID'], 
                $config['VTBPAY_CLIENT_SECRET'], 
                $config['VTBPAY_TEST_MODE'], 
                $config['VTBPAY_CLIENT_MERCHANT']
            );
        }

        return $this->vtb_api_obj;
    }


    /**
     * Действия выполняемые при установке.
     *
     * @return bool Результат установки модуля.
     */
    public function install() 
    {
        // Checking PHP Version
        if (version_compare(PHP_VERSION, '7.4.0', '<')) {
            $this->_errors[] = $this->l('Для работы платежного модуля "Платежная система ВТБ" требуется PHP версии 7.4.0 или выше.');
            return false;
        }

        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('Для использования этого модуля необходимо включить на сервере расширение cURL.');
            return false;
        }

        Configuration::updateValue('VTBPAY_TEST_MODE', true);

        return parent::install() && $this->registerHook('paymentOptions');
    }


    /**
     * Действия при удалении модуля.
     *
     * @return bool Результат удаления модуля.
     */
    public function uninstall() 
    {
        // Delete module config variable
        Configuration::deleteByName('VTBPAY_TEST_MODE');
        Configuration::deleteByName('VTBPAY_CLIENT_ID');
        Configuration::deleteByName('VTBPAY_CLIENT_SECRET');
        Configuration::deleteByName('VTBPAY_CLIENT_MERCHANT');
        Configuration::deleteByName('VTBPAY_LOGGING');
        Configuration::deleteByName('VTBPAY_SUCCESSFUL_PAYMENT_PAGE');
        Configuration::deleteByName('VTBPAY_FAILED_PAYMENT_PAGE');

        return parent::uninstall();
    }


    /**
     * Отображает настройки модуля в административной части.
     *
     * @return string HTML-код настроек модуля.
     */
    public function getContent() 
    {
        if (((bool)Tools::isSubmit('submitVtbpayModule')) == true) $this->postProcess();

        $this->context->smarty->assign('module_dir', $this->_path);

        return $this->renderForm();
    }
    
    
    /**
     * Генерирует форму настроек модуля.
     *
     * @return string HTML-код формы настроек.
     */
    protected function renderForm() 
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitVtbpayModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . 
                                '&configure=' . $this->name . 
                                '&tab_module=' . $this->tab . 
                                '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFormValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];

        return $helper->generateForm([$this->getConfigForm()]);
    }


    /**
     * Возвращает массив с полями настроек.
     *
     * @return array Массив с полями настроек.
     */
    protected function getConfigForm() 
    {
        return include 'config_form.php';
    }


    /**
     * Возвращает массив значений конфигурации модуля.
     *
     * @return array Массив значений конфигурации.
     */
    protected function getConfigFormValues() 
    {
        return [
            'VTBPAY_TEST_MODE' => Configuration::get('VTBPAY_TEST_MODE', false),
            'VTBPAY_CLIENT_ID' => Configuration::get('VTBPAY_CLIENT_ID', null),
            'VTBPAY_CLIENT_SECRET' => Configuration::get('VTBPAY_CLIENT_SECRET', null),
            'VTBPAY_CLIENT_MERCHANT' => Configuration::get('VTBPAY_CLIENT_MERCHANT', null),
            'VTBPAY_LOGGING' => Configuration::get('VTBPAY_LOGGING', false),
            'VTBPAY_SUCCESSFUL_PAYMENT_PAGE' => Configuration::get('VTBPAY_SUCCESSFUL_PAYMENT_PAGE', null),
            'VTBPAY_FAILED_PAYMENT_PAGE' => Configuration::get('VTBPAY_FAILED_PAYMENT_PAGE', null)
        ];
    }


    /**
     * Обновляет отправленные настройки модуля.
     */
    protected function postProcess() 
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }


    /**
     * Возвращает опции платежа для отображения на странице оплаты.
     *
     * @param array $params Параметры платежа.
     * @return array Массив опций платежа.
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active || !$this->checkCurrency($params['cart'])) {
            return;
        }
        
        $paymentOption = new PaymentOption();
        $paymentOption->setModuleName($this->name);
        $paymentOption->setCallToActionText($this->l('Платежная система ВТБ'));
        $paymentOption->setAction($this->context->link->getModuleLink($this->name, 'payment', [], true));
        $paymentOption->setLogo(Media::getMediaPath(__DIR__ . '/views/img/label.png'));

        return [$paymentOption];
    }
    
    
    /**
     * Проверяет, доступна ли валюта для данного заказа.
     *
     * @param object $cart Объект корзины.
     * @return bool Результат проверки валюты.
     */
    public function checkCurrency($cart) 
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
            return false;
        } else {
            return true;
        }
    }
    
    
    /**
     * Устанавливает объект логирования для модуля.
     */
    public function setVtbpayLogger(): void 
    {
        $logging = $this->configure['VTBPAY_LOGGING'];
        
        $fileLogger = new FileLogger(PrestaShopLoggerInterface::DEBUG);
        
        $fileLogger->setFilename(dirname(__DIR__, 2) . '/var/logs/vtbpay.log');
        
        $this->logger = VtbPayLogger::getInstance()
                        ->setOption('showCurrentDate', false)
                        ->setOption('showLogLevel', false)
                        ->setCustomRecording(function($message) use ($fileLogger) 
                        {
                            // Логирование ошибки
                            $fileLogger->logError($message);
                            
                            PrestaShopLogger::addLog(
                                explode('Context:', $message)[0],
                                PrestaShopLoggerInterface::ERROR,
                                1,
                                'VtbPay'
                            );
                            
                        }, VtbPayLogger::LOG_LEVEL_ERROR)
                        ->setCustomRecording(function($message) use ($fileLogger, $logging) 
                        {
                            // Логирование сообщения дебага
                            if ($logging) $fileLogger->logDebug($message);
                            
                        }, VtbPayLogger::LOG_LEVEL_DEBUG);
    }
}